Package: rtables
Title: Reporting Tables
Version: 0.6.16
Date: 2026-04-22
Authors@R: c(
    person("Gabriel", "Becker", , "gabembecker@gmail.com", role = "aut",
           comment = "Original creator of the package"),
    person("Adrian", "Waddell", , "adrian.waddell@gmail.com", role = "aut"),
    person("Daniel", "Sabanés Bové", , "daniel.sabanes_bove@rconis.com", role = "ctb"),
    person("Maximilian", "Mordig", , "maximilian_oliver.mordig@roche.com", role = "ctb"),
    person("Davide", "Garolini", , "davide.garolini@roche.com", role = "aut",
           comment = c(ORCID = "0000-0002-1445-1369")),
    person("Emily", "de la Rua", , "emilydelarua@gmail.com", role = "aut",
           comment = c(ORCID = "0009-0000-8738-5561")),
    person("Abinaya", "Yogasekaram", , "ayogasek@gmail.com", role = "ctb",
           comment = c(ORCID = "0009-0005-2083-1105")),
    person("Joe", "Zhu", , "joe.zhu@roche.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7566-2787")),
    person("F. Hoffmann-La Roche AG", role = c("cph", "fnd"))
  )
Description: Reporting tables often have structure that goes beyond simple
    rectangular data. The 'rtables' package provides a framework for
    declaring complex multi-level tabulations and then applying them to
    data. This framework models both tabulation and the resulting tables
    as hierarchical, tree-like objects which support sibling sub-tables,
    arbitrary splitting or grouping of data in row and column dimensions,
    cells containing multiple values, and the concept of contextual
    summary computations. A convenient pipe-able interface is provided for
    declaring table layouts and the corresponding computations, and then
    applying them to data.
License: Apache License 2.0 | file LICENSE
URL: https://github.com/insightsengineering/rtables,
        https://insightsengineering.github.io/rtables/
BugReports: https://github.com/insightsengineering/rtables/issues
Depends: formatters (>= 0.5.12), magrittr (>= 2.0.3), methods, R (>=
        4.1)
Imports: checkmate (>= 2.1.0), htmltools (>= 0.5.8.1), lifecycle (>=
        1.0.4), stats, stringi (>= 1.8.4)
Suggests: broom (>= 1.0.8), car (>= 3.1-3), dplyr (>= 1.0.5), knitr (>=
        1.42), lme4 (>= 1.1-35.5), r2rtf (>= 0.3.2), rmarkdown (>=
        2.28), survival (>= 3.3-1), testthat (>= 3.3.0), tibble (>=
        3.2.1), tidyr (>= 1.1.3), withr (>= 2.0.0), xml2 (>= 1.3.5)
VignetteBuilder: knitr, rmarkdown
Config/Needs/verdepcheck: insightsengineering/formatters,
        tidyverse/magrittr, mllg/checkmate, rstudio/htmltools,
        gagolews/stringi, tidymodels/broom, cran/car, tidyverse/dplyr,
        davidgohel/flextable, yihui/knitr, r-lib/lifecycle,
        davidgohel/officer, Merck/r2rtf, rstudio/rmarkdown,
        therneau/survival, r-lib/testthat, tidyverse/tibble,
        tidyverse/tidyr, r-lib/withr, r-lib/xml2
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Collate: '00tabletrees.R' 'Viewer.R' 'argument_conventions.R'
        'as_html.R' 'utils.R' 'colby_constructors.R'
        'compare_rtables.R' 'format_rcell.R' 'indent.R'
        'make_subset_expr.R' 'custom_split_funs.R'
        'default_split_funs.R' 'make_split_fun.R' 'summary.R'
        'package.R' 'tree_accessors.R' 'tt_afun_utils.R' 'tt_as_df.R'
        'tt_compare_tables.R' 'tt_compatibility.R' 'tt_dotabulation.R'
        'tt_paginate.R' 'tt_pos_and_access.R' 'tt_showmethods.R'
        'tt_sort.R' 'tt_test_afuns.R' 'tt_toString.R' 'tt_export.R'
        'index_footnotes.R' 'tt_from_df.R' 'validate_table_struct.R'
        'zzz_constants.R'
NeedsCompilation: no
Packaged: 2026-04-22 07:03:56 UTC; zhus31
Author: Gabriel Becker [aut] (Original creator of the package),
  Adrian Waddell [aut],
  Daniel Sabanés Bové [ctb],
  Maximilian Mordig [ctb],
  Davide Garolini [aut] (ORCID: <https://orcid.org/0000-0002-1445-1369>),
  Emily de la Rua [aut] (ORCID: <https://orcid.org/0009-0000-8738-5561>),
  Abinaya Yogasekaram [ctb] (ORCID:
    <https://orcid.org/0009-0005-2083-1105>),
  Joe Zhu [aut, cre] (ORCID: <https://orcid.org/0000-0001-7566-2787>),
  F. Hoffmann-La Roche AG [cph, fnd]
Maintainer: Joe Zhu <joe.zhu@roche.com>
Repository: CRAN
Date/Publication: 2026-04-22 08:30:02 UTC
Built: R 4.5.2; ; 2026-04-22 09:41:17 UTC; unix
