# Generated by roxygen2: do not edit by hand

S3method(print,CellValue)
S3method(print,RowsVerticalSection)
export("cell_footnotes<-")
export("clayout<-")
export("col_counts<-")
export("col_fnotes_here<-")
export("col_footnotes<-")
export("col_info<-")
export("col_total<-")
export("colcount_format<-")
export("colcount_na_str<-")
export("colcount_visible<-")
export("content_table<-")
export("facet_colcount<-")
export("facet_colcounts_visible<-")
export("fnotes_at_path<-")
export("header_section_div<-")
export("horizontal_sep<-")
export("indent_mod<-")
export("label_at_path<-")
export("obj_stat_names<-")
export("ref_index<-")
export("ref_symbol<-")
export("row_footnotes<-")
export("section_div<-")
export("section_div_at_path<-")
export("top_left<-")
export("top_level_section_div<-")
export("tree_children<-")
export("tt_at_path<-")
export(.add_row_summary)
export(.tablerow)
export(AnalyzeColVarSplit)
export(AnalyzeMultiVars)
export(AnalyzeVarSplit)
export(CellValue)
export(ContentRow)
export(DataRow)
export(ElementaryTable)
export(InstantiatedColumnInfo)
export(LabelRow)
export(ManualSplit)
export(MultiVarSplit)
export(TableTree)
export(VarDynCutSplit)
export(VarLevWBaselineSplit)
export(VarLevelSplit)
export(Viewer)
export(add_colcounts)
export(add_combo_facet)
export(add_combo_levels)
export(add_existing_table)
export(add_overall_col)
export(add_overall_facet)
export(add_overall_level)
export(add_to_split_result)
export(all_zero)
export(all_zero_or_na)
export(analyze)
export(analyze_colvars)
export(append_topleft)
export(as_html)
export(as_result_df)
export(assert_valid_table)
export(basic_table)
export(build_table)
export(cbind_rtables)
export(cell_footnotes)
export(cell_values)
export(clayout)
export(clear_indent_mods)
export(col_counts)
export(col_exprs)
export(col_fnotes_here)
export(col_footnotes)
export(col_info)
export(col_paths)
export(col_paths_summary)
export(col_total)
export(colcount_format)
export(colcount_na_str)
export(colcount_visible)
export(collect_leaves)
export(coltree)
export(coltree_structure)
export(compare_rtables)
export(cont_n_allcols)
export(cont_n_onecol)
export(content_all_zeros_nas)
export(content_table)
export(counts_wpcts)
export(df_to_tt)
export(do_base_split)
export(drop_and_remove_levels)
export(drop_facet_levels)
export(drop_split_levels)
export(export_as_pdf)
export(export_as_tsv)
export(export_as_txt)
export(facet_colcount)
export(find_degen_struct)
export(format_rcell)
export(get_formatted_cells)
export(has_force_pag)
export(head)
export(header_section_div)
export(horizontal_sep)
export(import_from_tsv)
export(in_rows)
export(indent)
export(indent_mod)
export(indent_string)
export(insert_row_at_path)
export(insert_rrow)
export(is_rtable)
export(keep_split_levels)
export(label_at_path)
export(list_wrap_df)
export(list_wrap_x)
export(low_obs_pruner)
export(make_afun)
export(make_col_df)
export(make_split_fun)
export(make_split_result)
export(make_subset_expr)
export(manual_cols)
export(no_colinfo)
export(non_ref_rcell)
export(obj_avar)
export(obj_stat_names)
export(pag_tt_indices)
export(paginate_table)
export(path_enriched_df)
export(prune_empty_level)
export(prune_table)
export(prune_zeros_only)
export(qtable)
export(qtable_layout)
export(rawvalues)
export(rbindl_rtables)
export(rcell)
export(ref_index)
export(ref_msg)
export(ref_symbol)
export(remove_split_levels)
export(reorder_split_levels)
export(restrict_facets)
export(rheader)
export(rm_all_colcounts)
export(row_cells)
export(row_footnotes)
export(row_paths)
export(row_paths_summary)
export(row_values)
export(rrow)
export(rrowl)
export(rtable)
export(rtablel)
export(sanitize_table_struct)
export(section_div)
export(section_div_at_path)
export(section_div_info)
export(select_all_levels)
export(simple_analysis)
export(sort_at_path)
export(spl_context_to_disp_path)
export(spl_variable)
export(split_cols_by)
export(split_cols_by_cutfun)
export(split_cols_by_cuts)
export(split_cols_by_multivar)
export(split_cols_by_quartiles)
export(split_rows_by)
export(split_rows_by_cutfun)
export(split_rows_by_cuts)
export(split_rows_by_multivar)
export(split_rows_by_quartiles)
export(str)
export(subset_cols)
export(summarize_row_groups)
export(table_shell)
export(table_shell_str)
export(table_structure)
export(tail)
export(top_left)
export(top_level_section_div)
export(tree_children)
export(trim_levels_in_facets)
export(trim_levels_in_group)
export(trim_levels_to_map)
export(trim_rows)
export(tt_at_path)
export(tt_normalize_row_path)
export(tt_row_path_exists)
export(update_ref_indexing)
export(validate_table_struct)
export(value_at)
export(value_formats)
export(value_names)
export(vars_in_layout)
exportClasses(ContentRow)
exportClasses(CumulativeCutSplit)
exportClasses(DataRow)
exportClasses(ElementaryTable)
exportClasses(InstantiatedColumnInfo)
exportClasses(LabelRow)
exportClasses(TableTree)
exportClasses(VarDynCutSplit)
exportClasses(VarLevelSplit)
exportClasses(VarStaticCutSplit)
exportMethods("[")
exportMethods("[<-")
exportMethods("cell_footnotes<-")
exportMethods("clayout<-")
exportMethods("col_counts<-")
exportMethods("col_fnotes_here<-")
exportMethods("col_footnotes<-")
exportMethods("col_info<-")
exportMethods("col_total<-")
exportMethods("colcount_format<-")
exportMethods("colcount_na_str<-")
exportMethods("colcount_visible<-")
exportMethods("content_table<-")
exportMethods("facet_colcount<-")
exportMethods("fnotes_at_path<-")
exportMethods("horizontal_sep<-")
exportMethods("main_footer<-")
exportMethods("main_title<-")
exportMethods("obj_format<-")
exportMethods("obj_label<-")
exportMethods("obj_na_str<-")
exportMethods("obj_name<-")
exportMethods("obj_round_type<-")
exportMethods("obj_stat_names<-")
exportMethods("prov_footer<-")
exportMethods("ref_index<-")
exportMethods("ref_symbol<-")
exportMethods("row_footnotes<-")
exportMethods("row_values<-")
exportMethods("subtitles<-")
exportMethods("table_inset<-")
exportMethods("top_left<-")
exportMethods("tree_children<-")
exportMethods("tt_at_path<-")
exportMethods(as.vector)
exportMethods(c)
exportMethods(cell_footnotes)
exportMethods(cell_values)
exportMethods(clayout)
exportMethods(clear_indent_mods)
exportMethods(col_counts)
exportMethods(col_fnotes_here)
exportMethods(col_footnotes)
exportMethods(col_info)
exportMethods(col_total)
exportMethods(colcount_format)
exportMethods(colcount_na_str)
exportMethods(colcount_visible)
exportMethods(collect_leaves)
exportMethods(coltree)
exportMethods(content_table)
exportMethods(dim)
exportMethods(facet_colcount)
exportMethods(has_force_pag)
exportMethods(head)
exportMethods(horizontal_sep)
exportMethods(length)
exportMethods(main_footer)
exportMethods(main_title)
exportMethods(make_row_df)
exportMethods(matrix_form)
exportMethods(names)
exportMethods(ncol)
exportMethods(nlines)
exportMethods(no_colinfo)
exportMethods(nrow)
exportMethods(obj_avar)
exportMethods(obj_format)
exportMethods(obj_label)
exportMethods(obj_na_str)
exportMethods(obj_name)
exportMethods(obj_round_type)
exportMethods(obj_stat_names)
exportMethods(prov_footer)
exportMethods(rbind)
exportMethods(rbind2)
exportMethods(ref_index)
exportMethods(ref_msg)
exportMethods(ref_symbol)
exportMethods(rm_all_colcounts)
exportMethods(row.names)
exportMethods(row_cells)
exportMethods(row_footnotes)
exportMethods(row_values)
exportMethods(simple_analysis)
exportMethods(spl_variable)
exportMethods(str)
exportMethods(subset_cols)
exportMethods(subtitles)
exportMethods(table_inset)
exportMethods(tail)
exportMethods(toString)
exportMethods(top_left)
exportMethods(tree_children)
exportMethods(tt_at_path)
exportMethods(value_at)
import(formatters)
import(methods)
importFrom(formatters,do_forced_paginate)
importFrom(formatters,export_as_pdf)
importFrom(formatters,export_as_txt)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%>%")
importFrom(stats,binom.test)
importFrom(stats,na.omit)
importFrom(stats,prop.test)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(stats,setNames)
importFrom(tools,file_ext)
importFrom(utils,browseURL)
importFrom(utils,compareVersion)
importFrom(utils,head)
importFrom(utils,head.matrix)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,tail.matrix)
importFrom(utils,write.table)
importMethodsFrom(formatters,matrix_form)
importMethodsFrom(formatters,nlines)
importMethodsFrom(formatters,toString)
